using System;
using System.Drawing;
using MonoTouch.Foundation;
using MonoTouch.UIKit;
using Infragistics;
using System.Collections.Generic;
using System.Linq;

namespace TestInfragistics
{
    public partial class TestInfragisticsViewController : UIViewController
    {
        public IGGridView Grid { get; set; }

        public TestInfragisticsViewController(IntPtr handle) : base (handle)
        {

        }

        public override void ViewDidLoad()
        {
            base.ViewDidLoad();

            Grid = new IGGridView(View.Bounds, IGGridViewStyle.IGGridViewStyleDefault);
            Grid.AutoresizingMask = ~UIViewAutoresizing.None;
            Grid.DataSource = new GridDataSource(this);
            Grid.SelectionType = IGGridViewSelectionType.IGGridViewSelectionTypeCell;
            Grid.AllowsMultipleSelection = false;
            Grid.HeaderHeight = 120;
            Grid.Delegate = new GridViewDelegate(this);

            View.AddSubview(Grid);
        }
    }

    public class GridDataSource : IGGridViewDataSource
    {
        private TestInfragisticsViewController _controller;
        private List<string> _columns;

        public GridDataSource(TestInfragisticsViewController controller)
        {
            _controller = controller;
            _columns = Enumerable.Range(0, 20).Select(i => string.Concat("Spalte\n-\n", i)).ToList();
        }

        public override int NumberOfColumns(IGGridView gridView)
        {
            return _columns.Count;
        }

        public override int NumberOfRowsInSection(IGGridView gridView, int section)
        {
            return 100;
        }

        public override int NumberOfSections(IGGridView gridView)
        {
            return 1;
        }

        public override int NumberOfFixedLeftColumns(IGGridView gridView)
        {
            return 4;
        }

        public override IGGridViewCell CreateCell(IGGridView gridView, IGCellPath path)
        {
            var textCell = gridView.DequeueReusableCell("numberCell") as IGGridViewCell;
            if (textCell == null)
            {
                textCell = new IGGridViewCell("numberCell");
            }

            textCell.TextLabel.Text = string.Format("{0}.{1}", path.ColumnIndex, path.RowIndex);
            textCell.BackgroundColor = UIColor.White;
            return textCell;           
        }

        public override IGColumnWidth GetWidthForFixedLeftColumn(IGGridView gridView, int column)
        {
            switch (column)
            {
                case 0:
                    return IGColumnWidth.CreateNumericColumnWidth(50);
                case 1:
                    return IGColumnWidth.CreateNumericColumnWidth(100);
                case 2:
                    return IGColumnWidth.CreateNumericColumnWidth(200);
                case 3:
                    return IGColumnWidth.CreateNumericColumnWidth(50);
                default:
                    throw new InvalidOperationException();
            }
        }

        public override IGColumnWidth ResolveColumnWidth(IGGridView gridView, int column)
        {
            return IGColumnWidth.CreateNumericColumnWidth(75);
        }

        public override string TitleForFixedLeftColumnHeader(IGGridView gridView, int column)
        {
            switch (column)
            {
                case 0:
                    return string.Empty;
                case 1:
                    return "Dokument";
                case 2:
                    return "Bezeichnung";
                case 3:
                    return "Meh";
                default:
                    throw new InvalidOperationException();
            }
        }

        public override IGGridViewCell CreateFixedLeftCell(IGGridView gridView, IGCellPath path)
        {
            IGGridViewCell cell = null;

            if (path.ColumnIndex == 0)
            {
                var menuCell = gridView.DequeueReusableCell("menuCell") as IGGridViewImageCell;
                if (menuCell == null)
                {
                    menuCell = new IGGridViewImageCell("menuCell");
                }

                menuCell.ImageView.Image = UIImage.FromBundle("Images/text-list.png");
                cell = menuCell;
            }
            else if (path.ColumnIndex >= 1 && path.ColumnIndex <= 2)
            {
                var textCell = gridView.DequeueReusableCell("textCell") as IGGridViewCell;
                if (textCell == null)
                {
                    textCell = new IGGridViewCell("textCell");
                }

                textCell.TextLabel.Text = string.Format("{0}.{1}", path.ColumnIndex, path.RowIndex);
                cell = textCell;
            }
            else if (path.ColumnIndex == 3)
            {
                var textCell = gridView.DequeueReusableCell("textCell") as IGGridViewCell;
                if (textCell == null)
                {
                    textCell = new IGGridViewCell("textCell");
                }

                textCell.TextLabel.Text = "kg";
                cell = textCell;
            }

            cell.BackgroundColor = UIColor.White;
            return cell;
        }

        public override string TitleForHeaderInColumn(IGGridView gridView, int column)
        {
            return _columns[column];
        }

        public override IGGridViewHeaderCell CreateHeaderCell(IGGridView gridView, int column)
        {
            var headerCell = new IGGridViewHeaderCell("headerCell");
            headerCell.TextLabel.Text = TitleForHeaderInColumn(gridView, column);
            headerCell.TextLabel.TextAlignment = UITextAlignment.Center;


            return headerCell;
        }

    }

    public class GridViewDelegate : IGGridViewDelegate
    {
        private TestInfragisticsViewController _controller;

        public GridViewDelegate(TestInfragisticsViewController controller)
        {
            _controller = controller;
        }

        public override void DidSelectCell(IGGridView gridView, IGCellPath path)
        {
            if (path.ColumnIndex == 0 && path.IsFixed == IGGridViewFixedColumnDirection.IGGridViewFixedColumnDirectionLeft)
                gridView.DeselectCell(path, false);
        }

        public override void InitializeCell(IGGridView gridView, IGGridViewCell cell)
        {
            var path = gridView.ResolvePathForCell(cell);

            if (path.IsFixed == IGGridViewFixedColumnDirection.IGGridViewFixedColumnDirectionNone)
            {
                var tapGesture = new UITapGestureRecognizer(HandleInput) { NumberOfTapsRequired = 1};

                cell.RegisterGestures(new[] { tapGesture });
            }
        }

        private void HandleInput(UITapGestureRecognizer gestureRecognizer)
        {
            var cell = gestureRecognizer.View as IGGridViewCell;

            var viewController = new UIViewController();
            var textField = new UITextField(viewController.View.Bounds) { AutoresizingMask = ~UIViewAutoresizing.None, Text = cell.TextLabel.Text };
            viewController.View.AddSubview(textField);
            var popover = new UIPopoverController(viewController);
            popover.PresentFromRect(cell.Bounds, cell, UIPopoverArrowDirection.Any, true);
        }
    }
}

